'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /**
                                                                                                                                                                                                                                                                   * data-snapshot
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * generic thunk api for storing promise data in a __data_snapshot__ file
                                                                                                                                                                                                                                                                   * for use in test to store real mock data
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * 
                                                                                                                                                                                                                                                                   */


var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var shouldUpdate = function shouldUpdate() {
  var argv = process.argv.slice(2);
  return ['--updateSnapshot', '-u'].some(function (d) {
    return argv.includes(d);
  });
};

var banner = '// Data Snap v1';

var ensureDirectoryExistence = function ensureDirectoryExistence(filePath) {
  var dirname = _path2.default.dirname(filePath);
  if (_fs2.default.existsSync(dirname)) {
    return true;
  }

  ensureDirectoryExistence(dirname);
  _fs2.default.mkdirSync(dirname);
};

var dataSnapshot = function dataSnapshot(dataKey, thunkPromise) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var callingFile = (0, _utils.callingFilePath)(new Error().stack);
  var relativeCallingFile = _path2.default.relative(process.cwd(), callingFile);

  var basename = _path2.default.basename(callingFile) + '.snap';
  var snapFile = _path2.default.resolve(callingFile, '..', '__data_snapshots__', basename);

  var exports = {};

  if (_fs2.default.existsSync(snapFile)) {
    // Read a .snap file and assign the exports to our exports var
    eval(_fs2.default.readFileSync(snapFile, 'utf8'));
  }

  var settings = _extends({
    expires: Infinity
  }, opts);

  var _ref = exports[dataKey] || {},
      snapData = _ref.data,
      snapMeta = _ref.metadata;

  if (snapData && !shouldUpdate()) {
    if (snapMeta && snapMeta.expires && snapMeta.expires < Date.now()) {
      // eslint-disable-next-line no-console
      console.log(_chalk2.default.red('Data "' + dataKey + '" in ' + relativeCallingFile + ' is expired - please update!'));
    }

    return Promise.resolve(snapData);
  }

  // eslint-disable-next-line no-console
  console.log(_chalk2.default.yellow('Calling thunk promise:\n') + ('-- file: ' + relativeCallingFile + '\n-- key: ' + dataKey));

  return thunkPromise().then(function (data) {
    exports[dataKey] = {
      data: data,
      metadata: {
        expires: settings.expires
      }
    };

    var newSnap = Object.keys(exports).sort().reduce(function (acc, key) {
      return acc + ('\nexports[\'' + key + '\'] = ' + JSON.stringify(exports[key], null, 2) + '\n');
    }, banner + '\n');

    ensureDirectoryExistence(snapFile);
    _fs2.default.writeFileSync(snapFile, newSnap);
    return data;
  });
};

exports.default = dataSnapshot;